/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.common.ReportWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.MessageDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.navigation", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var NavigationMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.navigation", "Messages");
var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var DROPDOWN_HOVER			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/dd-arrow-hov.gif");
var DROPDOWN				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/dd-arrow.gif");
var REPORT_DESIGN			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design.gif");
var REPORT_QUERY			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_query.gif");
var REPORT_SNAPSHOT			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_snapshot.gif");
var REPORT_DESIGN_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design_shared.gif");
var REPORT_QUERY_SHARED		= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_query_shared.gif");
var REPORT_SNAPSHOT_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_snapshot_shared.gif");

/**
 * Widget to display a report node.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.common.ReportWidget",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/templates/ReportWidget.html"),

	RUN_REPORT_ICON_CLASS: "runReportIcon",
	ADD_TO_FOLDER_ICON: "addToToFolderIcon",
	REMOVE_REPORT_ICON: "removeReportIcon",
	DELETE_ICON_CLASS: "deleteReportIcon",
	REPORT_PROPERTIES_ICON: "reportPropertiesIcon",
		
	postCreate: function() {
		this._reportIcon.src = REPORT_DESIGN;
		this._dropDownImage.src = DROPDOWN;
		this._report = this.report;
		this._controller = this.controller;
		this._parentWidget = this.parentWidget;
		
		this._dropDownImage.alt = CommonMessages.OPEN_ALT_LABEL;
		
		var type = this.report.report.type;
		
		this._textNode=null;
		this._menu = null;
		this._isMenuOpen = false;
		
		this._setIcon();		
		this._setReport(this._report);
		
		var self = this;
		dojo.connect(this._compactReportTable, "onfocus", function(e) {
			e.preventDefault();
			self._dropDown.style.visibility = "visible";
			self._dropDown.style.display = "inline";
			self._dropDownImage.src = DROPDOWN;
		});
		
		dojo.connect(this._compactReportTable, "onblur", function(e) {
			e.preventDefault();
			if (!self._isMenuOpen) {
				self._dropDown.style.visibility = "hidden";
				self._dropDown.style.display = "none";
				self._dropDownImage.src = DROPDOWN;
			}
		});
			
		dojo.connect(this._dropDown, "onfocus", null, function(e) {
			e.preventDefault();
			self._dropDownImage.src = DROPDOWN_HOVER;	
		});
		
		dojo.connect(this._dropDown, "onblur", null, function(e) {
			e.preventDefault();
			if (!self._isMenuOpen) {
				self._dropDown.style.visibility = "hidden";
				self._dropDown.style.display = "none";
			}
		});
		
		dojo.connect(document, "onkeypress", function(e) {
			if (e.keyCode == 27 && self._isMenuOpen) {
				self._closeMenu();
				self._dropDownImage.src = DROPDOWN_HOVER;
				self._dropDown.focus();
			}
		});
		
		if (!dojo.isIE) {
			this._reportIcon.className = "report-icon-mozilla";
		}		
	},

	_updateNode: function(report) {
		this._reportLinkTd.removeChild(this._reportLinkTd.lastChild);
		this._report = report;
		this._setIcon();
		this._createReportLink();		
	},
	
	
	_setIcon: function() {
		var currentImage = this._report.shared ? REPORT_DESIGN_SHARED : REPORT_DESIGN;
		if (this._report.nodeType == "REPORT_QUERY") {
			currentImage = this._report.shared ? REPORT_QUERY_SHARED : REPORT_QUERY;	
		} else if (this._report.nodeType == "REPORT_SNAPSHOT") {
			currentImage = this._report.shared ? REPORT_SNAPSHOT_SHARED : REPORT_SNAPSHOT;
		}
		this._reportIcon.src = currentImage;
	},
	
	_createReportLink: function() {
		var self = this;
		var anchor = document.createElement("a");
		anchor.className = "report-link";
		anchor.href = "javascript:{}";
		anchor.onclick = function() {
			self._runReport();
		};
		anchor.title = this._report.name;
		
		if (dojo.isIE) {
			anchor.attachEvent('onfocus', function(evt) {
				self._dropDown.style.visibility = "visible";
				self._dropDown.style.display = "inline";
				self._dropDownImage.src = DROPDOWN;
			});
		} else if (dojo.isMozilla) {
			anchor.addEventListener('focus', function(evt) {
				self._dropDown.style.visibility = "visible";
				self._dropDown.style.display = "inline";
				self._dropDownImage.src = DROPDOWN;
			}, false);
		}
		
		// Store the text node in a variable so we can change it if the 
		// folder gets renamed.
		this._textNode = document.createTextNode(this._report.name);
		anchor.appendChild(this._textNode);
		this._reportLinkTd.appendChild(anchor);
	},
	
	/**
	 * Create the report contents.
	 */
	_setReport: function(report) {
		report.div = this.reportDiv;
        this._setReportLink(this.reportDiv, report);
	},

	/**
	 * Adds a link for the report to the list.
	 */
	_setReportLink: function (parentDiv, report) {
		// For access to "this" in anonymous functions
		var self = this;
		
		this._createReportLink();
		
		
		
		if (!dojo.isIE) {
			var itemRowDivTopLeft = document.createElement("span");
			itemRowDivTopLeft.className = "itemRowDivTopLeft";
			var itemRowDivTopRight = document.createElement("span");
			itemRowDivTopRight.className = "itemRowDivTopRight";
			this._treeItem.insertBefore(itemRowDivTopRight, this._treeItem.firstChild);
			this._treeItem.insertBefore(itemRowDivTopLeft, this._treeItem.firstChild);
		}

		// Hide the drop-down indicator initially
		this._dropDown.style.visibility = "hidden";
		this._dropDown.style.display = "none";

		// Hook the events for the drop-down indicator
		dojo.connect(this._dropDownImage, "onmouseover", null,
			function(e) {
				e.preventDefault();
				self._dropDownImage.src = DROPDOWN_HOVER;
			}
		);
		dojo.connect(this._dropDownImage, "onmouseout", null,
			function(e) {
				e.preventDefault();
				self._dropDownImage.src = DROPDOWN;
			}
		);
		dojo.connect(this._dropDown, "onclick", null,
			function(e) {
				e.preventDefault();
				self._dropDownImage.src = DROPDOWN;
				self._openMenu(e);
			}
		);
		
		if (!dojo.isIE) {
			var itemRowDivBottomLeft = document.createElement("span");
			itemRowDivBottomLeft.className = "itemRowDivBottomLeft";
			var itemRowDivBottomRight = document.createElement("span");
			itemRowDivBottomRight.className = "itemRowDivBottomRight";
			this._treeItem.appendChild(itemRowDivBottomLeft);
			this._treeItem.appendChild(itemRowDivBottomRight);
		}			
		
		// Blue hover support for the whole item
		dojo.connect(this._itemDiv, "onmouseover", null,
			function(e) {
				e.preventDefault();
				if (self._isMenuOpen) {
					// We're already in the menu, ignore this
					self._mouseOutOccurred = false;
					return;
				}
				self._treeItem.className = "treeItemSelected";
				self._dropDown.style.visibility = "visible";
				self._dropDown.style.display = "inline";
			}
		);
		dojo.connect(this._itemDiv, "onmouseout", null,
			function(e) {
				e.preventDefault();
				if (self._isMenuOpen) {
					// We're in the menu, don't remove the hover
					// but remember that we left
					self._mouseOutOccurred = true;
					return;
				}
				if (self._isSelected) {
					self._treeItem.className = "treeItemSelected";
				} else {
					self._treeItem.className = "treeItem";
				}
				self._dropDown.style.visibility = "hidden";
				self._dropDown.style.display = "none";
			}
		);
	},
	
	_runReport: function() {
		var args = new Object();
		args.id = this._report.report.itemId;
		if (this._report.nodeType == "REPORT_DESIGN") {
			this._controller.viewReportDesign(args);
		}
		if (this._report.nodeType == "REPORT_QUERY") {
			this._controller.viewReportQuery(args);
		}
		if (this._report.nodeType == "REPORT_SNAPSHOT") {
			this._controller.viewReportSnapshot(args);				
		}
	},
	
	_deleteFromTree: function() {
		this._controller.deleteReportNode(this._report, false);
	},
	
	_delete: function() {
		this._controller.deleteReportNode(this._report, true);
	},
	
	_properties: function() {
		var reportId = this._report.report.itemId;
				
		var srh = new ServiceResponseHandler(this, "_getReportSuccess", "_getReportError");						
		ReportClient.getReport(srh, {id:reportId});				
	},
	
	_getReportSuccess: function(report) {
		this._controller.editReportNode(this._report, report);
	},
	
	_getReportError: function(error) {

	},
	
	_openMenu: function(e) {
		if (this._isMenuOpen) {
			this._closeMenu();
			return;
		}
		
		if (!this._menu) {
			var editable = this._controller.secMgr.isOwner(this._report) || this._controller.secMgr.isAdmin();
			var menu = new dijit.Menu();
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.RUN_MENU_ITEM_LABEL,
				iconClass: this.RUN_REPORT_ICON_CLASS,
				onClick: dojo.hitch(this, this._runReport)
			}));
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.REMOVE_FROM_FOLDER_MENU_ITEM_LABEL,
				iconClass: this.REMOVE_REPORT_ICON,
				onClick: dojo.hitch(this, this._displayConfirmationMessageRemoval),
				disabled: !editable
			}));
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.DELETE_MENU_ITEM_LABEL,
				iconClass: this.DELETE_ICON_CLASS,
				onClick: dojo.hitch(this, this._displayConfirmationMessage),
				disabled: !editable
			}));
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.PROPERTIES_MENU_ITEM_LABEL,
				iconClass: this.REPORT_PROPERTIES_ICON,
				onClick: dojo.hitch(this, this._properties)
			}));
			this._menu = menu;
		}
		var coords = dojo.coords(e.target, true);
		dijit.popup.open({ popup: this._menu, parent: this, x: coords.x + 10, y: coords.y + 10, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this._menu,"_onBlur",this._closeMenu);
		this._isMenuOpen = true;
		this._menu.focus();			
	},
	
	_displayConfirmationMessage: function(){
		var self = this;
		var prompt;
		if (this._report.nodeType == "REPORT_DESIGN") {
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORT_DESIGN;
		}
		if (this._report.nodeType == "REPORT_QUERY") {
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORT_QUERY;
		}
		if (this._report.nodeType == "REPORT_SNAPSHOT") {
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORT_SNAPSHOT;				
		}
		var confirmationDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.CONFIRMATION, prompt: prompt, promptType: "Warning", buttonType: "YesNo"});
		dojo.connect(confirmationDialog.leftButton, "onclick", confirmationDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._delete();
				confirmationDialog.close();
			}
		);	
		dojo.connect(confirmationDialog.rightButton, "onclick", confirmationDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				confirmationDialog.close();
			}
		);
	},
	
	_displayConfirmationMessageRemoval: function(){
		var self = this;
		var prompt;
		if (this._report.nodeType == "REPORT_DESIGN") {
			prompt = CommonMessages.CONFIRMATION_REMOVE_REPORT_DESIGN;
		}
		if (this._report.nodeType == "REPORT_QUERY") {
			prompt = CommonMessages.CONFIRMATION_REMOVE_REPORT_QUERY;
		}
		if (this._report.nodeType == "REPORT_SNAPSHOT") {
			prompt = CommonMessages.CONFIRMATION_REMOVE_REPORT_SNAPSHOT;				
		}
		var confirmationDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.CONFIRMATION, prompt: prompt, promptType: "Warning", buttonType: "YesNo"});
		dojo.connect(confirmationDialog.leftButton, "onclick", confirmationDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._deleteFromTree();
				confirmationDialog.close();
			}
		);	
		dojo.connect(confirmationDialog.rightButton, "onclick", confirmationDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				confirmationDialog.close();
			}
		);
	},
	
	_closeMenu: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._mouseOutOccurred) {
			if (this._isSelected) {
				this._treeItem.className = "treeItemSelected";
			} else {
				this._treeItem.className = "treeItem";
			}
			this._dropDown.style.visibility = "hidden";
			this._dropDown.style.display = "none";
		}
		this.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._isMenuOpen = false;
	},
		
	_trim: function(string) {
		return this._trimLeft(this._trimRight(string));
	},
	
	_trimLeft: function(string) {
		return string.replace(new RegExp("^[\\s]+", "g"), "");
	},
	
	_trimRight: function(string) {
		return string.replace(new RegExp("[\\s]+$", "g"), "");
	}
	
});

})();