/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.EditPasswordViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.report.web.core.client.internal.UserClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");

var CommonMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var CoreAdminMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
var UserClient				= com.ibm.rational.report.web.core.client.internal.UserClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Displays the Change Password view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.EditPasswordViewer",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/EditPasswordViewer.html"),
	
	postCreate: function() {
		
		Util.removeChildren(this.pageTitle);
		this.pageTitle.appendChild(document.createTextNode(CoreAdminMessages.EDITPASSWORD_PAGE_TITLE));
		
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		Util.removeChildren(this.editOldPasswordLabel);
		this.editOldPasswordLabel.appendChild(document.createTextNode(CoreAdminMessages.EDITPASSWORD_EDIT_OLD_PASSWORD_LABEL));
		
		Util.removeChildren(this.editNewPasswordLabel);
		this.editNewPasswordLabel.appendChild(document.createTextNode(CoreAdminMessages.EDITPASSWORD_EDIT_NEW_PASSWORD_LABEL));
		
		Util.removeChildren(this.editConfirmPasswordLabel);
		this.editConfirmPasswordLabel.appendChild(document.createTextNode(CoreAdminMessages.EDITPASSWORD_CONFIRM_PASSWORD_LABEL));
				
		this._msgFormatter = new MessageFormatter(this.messageArea);

		var self = this;

		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._validateUserInput();
			}
		);
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				self._exitPage();
			}
		);
	},
	
	constructor: function(args) {
		this._controller = args.controller;
		this._userName = args.userName;
		this._editUserProfile = args.editUserProfile;
	
	},

	_updatePassword: function(result) {
		var responseHandler = new ServiceResponseHandler(
			this,
			"_exitPage",
			"_postUpdatePasswordError");

		if (result == "true") {
			var args = {
				name: this._userName,
				password: this.editNewPassword.value
			};
			UserClient.postUpdateUserAttributes(responseHandler, args);
		}
		else {
			this._msgFormatter.displayErrorMessage(ServiceabilityMessages.INCORRECT_OLD_PASSWORD, null);
		}
	},

	_validateUserInput: function() {	
		if (dojo.trim(this.editOldPassword.value).length == 0 ||		
			dojo.trim(this.editNewPassword.value).length == 0 ||
			dojo.trim(this.editConfirmPassword.value).length == 0) {
			this._msgFormatter.displayErrorMessage(ServiceabilityMessages.ALL_FIELDS_ARE_REQUIRED, null);
			this.saveButton.disabled = true;
		}
		else if (dojo.trim(this.editNewPassword.value) !=
				 dojo.trim(this.editConfirmPassword.value)) {
			this._msgFormatter.displayErrorMessage(ServiceabilityMessages.MISMATCH_PASSWORDS, null);
			this.saveButton.disabled = true;
		}
		else if (dojo.trim(this.editOldPassword.value) ==
				 dojo.trim(this.editNewPassword.value)) {
			this._msgFormatter.displayErrorMessage(ServiceabilityMessages.IDENTICAL_PASSWORDS, null);
			this.saveButton.disabled = true;
		}
		else {
			this.saveButton.disabled = false;

			var responseHandler = new ServiceResponseHandler(
				this,
				"_updatePassword",
				"_getValidatePasswordError");

			var args = {
				name: this._userName,
				password: this.editOldPassword.value
			};
			UserClient.getValidatePassword(responseHandler, args);
		}
	},				

	_getValidatePasswordError: function(error) {
			var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_RETRIEVE_PASSWORD,[this._userName]);
			this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	_postUpdatePasswordError: function(error) {
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_USER,[this._userName]);
		this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	_exitPage: function() {
		var args = {
			controller: this._controller,
			userName : this._userName,				
			editUserProfile: this._editUserProfile
		};
		this._controller.editUser(args);
	}

});

})();